package com.sun.org.apache.xpath.internal;

import java.io.PrintWriter;
import java.io.PrintStream;
import org.w3c.dom.Node;
import javax.xml.transform.TransformerException;

public class XPathException extends TransformerException
{
    static final long serialVersionUID = 4263549717619045963L;
    Object m_styleNode;
    protected Exception m_exception;
    
    public Object getStylesheetNode() {
        return this.m_styleNode;
    }
    
    public void setStylesheetNode(Object styleNode) {
        this.m_styleNode = styleNode;
    }
    
    public XPathException(String message, ExpressionNode ex) {
        super(message);
        this.m_styleNode = null;
        this.setLocator(ex);
        this.setStylesheetNode(this.getStylesheetNode(ex));
    }
    
    public XPathException(String message) {
        super(message);
        this.m_styleNode = null;
    }
    
    public Node getStylesheetNode(ExpressionNode ex) {
        ExpressionNode owner = this.getExpressionOwner(ex);
        if (null != owner && owner instanceof Node) {
            return (Node)owner;
        }
        return null;
    }
    
    protected ExpressionNode getExpressionOwner(ExpressionNode ex) {
        ExpressionNode parent;
        for (parent = ex.exprGetParent(); null != parent && parent instanceof Expression; parent = parent.exprGetParent()) {}
        return parent;
    }
    
    public XPathException(String message, Object styleNode) {
        super(message);
        this.m_styleNode = null;
        this.m_styleNode = styleNode;
    }
    
    public XPathException(String message, Node styleNode, Exception e) {
        super(message);
        this.m_styleNode = null;
        this.m_styleNode = styleNode;
        this.m_exception = e;
    }
    
    public XPathException(String message, Exception e) {
        super(message);
        this.m_styleNode = null;
        this.m_exception = e;
    }
    
    @Override
    public void printStackTrace(PrintStream s) {
        if (s == null) {
            s = System.err;
        }
        try {
            super.printStackTrace(s);
        }
        catch (Exception ex) {}
        Throwable exception = this.m_exception;
        for (int i = 0; i < 10 && null != exception; ++i) {
            s.println("---------");
            exception.printStackTrace(s);
            if ((exception instanceof TransformerException)) {
                TransformerException se = (TransformerException)exception;
                Throwable prev = exception;
                exception = se.getException();
                if (prev == exception) {
                    break;
                }
            }
            else {
                exception = null;
            }
        }
    }
    
    @Override
    public String getMessage() {
        String lastMessage = super.getMessage();
        Throwable exception = this.m_exception;
        while (null != exception) {
            String nextMessage = exception.getMessage();
            if (null != nextMessage) {
                lastMessage = nextMessage;
            }
            if ((exception instanceof TransformerException)) {
                TransformerException se = (TransformerException)exception;
                Throwable prev = exception;
                exception = se.getException();
                if (prev == exception) {
                    break;
                }
                continue;
            }
            else {
                exception = null;
            }
        }
        return (null != lastMessage) ? lastMessage : "";
    }
    
    @Override
    public void printStackTrace(PrintWriter s) {
        if (s == null) {
            s = new PrintWriter(System.err);
        }
        try {
            super.printStackTrace(s);
        }
        catch (Exception ex) {}
        boolean isJdk14OrHigher = false;
        try {
            Throwable.class.getMethod("getCause", (Class<?>[])null);
            isJdk14OrHigher = true;
        }
        catch (NoSuchMethodException ex2) {}
        if (!isJdk14OrHigher) {
            Throwable exception = this.m_exception;
            for (int i = 0; i < 10 && null != exception; ++i) {
                s.println("---------");
                try {
                    exception.printStackTrace(s);
                }
                catch (Exception e) {
                    s.println("Could not print stack trace...");
                }
                if ((exception instanceof TransformerException)) {
                    TransformerException se = (TransformerException)exception;
                    Throwable prev = exception;
                    exception = se.getException();
                    if (prev == exception) {
                        exception = null;
                        break;
                    }
                }
                else {
                    exception = null;
                }
            }
        }
    }
    
    @Override
    public Throwable getException() {
        return this.m_exception;
    }
}
